'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var bulb = require('bulb');

function keys(target, options) {
  options = options || {
    preventDefault: false
  };
  return new bulb.Signal(function (emit) {
    var handler = function handler(e) {
      if (options.preventDefault) {
        e.preventDefault();
      }

      emit.value(parseInt(e.keyCode));
    };

    target.addEventListener('keydown', handler, true);
    return function () {
      return target.removeEventListener('keydown', handler, true);
    };
  });
}

function state(target, options) {
  options = options || {
    preventDefault: false
  };
  return new bulb.Signal(function (emit) {
    var state = new Set();

    var downHandler = function downHandler(e) {
      if (options.preventDefault) {
        e.preventDefault();
      }

      var key = parseInt(e.keyCode);

      if (!state.has(key)) {
        state.add(key);
        emit.value(Array.from(state));
      }
    };

    var upHandler = function upHandler(e) {
      if (options.preventDefault) {
        e.preventDefault();
      }

      var key = parseInt(e.keyCode);

      if (state.has(key)) {
        state.delete(key);
        emit.value(Array.from(state));
      }
    };

    target.addEventListener('keydown', downHandler, true);
    target.addEventListener('keyup', upHandler, true);
    return function () {
      target.removeEventListener('keydown', downHandler, true);
      target.removeEventListener('keyup', upHandler, true);
    };
  });
}

function buttons(target, options) {
  options = options || {
    preventDefault: false
  };
  return new bulb.Signal(function (emit) {
    var handler = function handler(e) {
      if (options.preventDefault) {
        e.preventDefault();
      }

      emit.value(e.buttons);
    };

    target.addEventListener('mousedown', handler, true);
    return function () {
      return target.removeEventListener('mousedown', handler, true);
    };
  });
}

function position(target, options) {
  options = options || {
    preventDefault: false
  };
  return new bulb.Signal(function (emit) {
    var handler = function handler(e) {
      if (options.preventDefault) {
        e.preventDefault();
      }

      emit.value([e.clientX, e.clientY]);
    };

    target.addEventListener('mousemove', handler, true);
    return function () {
      return target.removeEventListener('mousemove', handler, true);
    };
  });
}

function state$1(target, options) {
  options = options || {
    preventDefault: false
  };
  return new bulb.Signal(function (emit) {
    var handler = function handler(e) {
      if (options.preventDefault) {
        e.preventDefault();
      }

      emit.value(e);
    };

    target.addEventListener('mousemove', handler, true);
    target.addEventListener('mousedown', handler, true);
    target.addEventListener('mouseup', handler, true);
    return function () {
      target.removeEventListener('mousemove', handler, true);
      target.removeEventListener('mousedown', handler, true);
      target.removeEventListener('mouseup', handler, true);
    };
  });
}

/**
 * This module provides functions to create signals that wrap the keyboard
 * input device.
 */

var Keyboard = {
  /**
   * Creates a signal that emits a value if a key is pressed on the `target` DOM
   * element.
   *
   * If a key is held down continuously, then the signal will repeatedly emit
   * values at a rate determined by your OS key repeat setting.
   *
   * @param {EventTarget} target The event target (e.g. a DOM element).
   * @param {Object} [options] The options.
   * @param {Booelan} [options.preventDefault=false] A boolean indicating whether
   * the default action should be taken for the event.
   * @returns {Signal} A new signal.
   * @example
   *
   * import { Keyboard } from 'bulb-input'
   *
   * const s = Keyboard.keys(document)
   *
   * s.subscribe(console.log) // 1, 2, ...
   */
  keys: function keys$$1(target, options) {
    return keys(target, options);
  },

  /**
   * Creates a signal that emits a value if the keyboard state changes.
   *
   * When a key is pressed or released, then the signal will emit an array
   * containing the key codes of all the currently pressed keys.
   *
   * @param {EventTarget} target The event target (e.g. a DOM element).
   * @param {Object} [options] The options.
   * @param {Booelan} [options.preventDefault=false] A boolean indicating whether
   * the default action should be taken for the event.
   * @returns {Signal} A new signal.
   * @example
   *
   * import { Keyboard } from 'bulb-input'
   *
   * const s = Keyboard.state(document)
   *
   * s.subscribe(console.log) // [1], [1, 2], ...
   */
  state: function state$$1(target, options) {
    return state(target, options);
  }
};
/**
 * This module provides functions to creating signals that wrap the mouse input
 * device.
 */

var Mouse = {
  /**
   * Creates a signal that emits a value if a mouse button is pressed.
   *
   * When a mouse button is pressed, then the signal will emit an integer
   * representing the logical sum of the currently pressed button codes (left=1,
   * right=2, middle=4).
   *
   * @param {EventTarget} target The event target (e.g. a DOM element).
   * @param {Object} [options] The options.
   * @param {Booelan} [options.preventDefault=false] A boolean indicating whether
   * the default action should be taken for the event.
   * @returns {Signal} A new signal.
   * @example
   *
   * import { mouse } from 'bulb-input'
   *
   * const s = Mouse.buttons(document)
   *
   * s.subscribe(console.log) // 1, 2, ...
   */
  buttons: function buttons$$1(target, options) {
    return buttons(target, options);
  },

  /**
   * Creates a signal that emits a value if the mouse is moved.
   *
   * When the mouse is moved, then the signal will emit an array containing the
   * mouse position.
   *
   * @param {EventTarget} target The event target (e.g. a DOM element).
   * @param {Object} [options] The options.
   * @param {Booelan} [options.preventDefault=false] A boolean indicating whether
   * the default action should be taken for the event.
   * @returns {Signal} A new signal.
   * @example
   *
   * import { mouse } from 'bulb-input'
   *
   * const s = Mouse.position(document)
   *
   * s.subscribe(console.log) // [1, 1], [2, 2], ...
   */
  position: function position$$1(target, options) {
    return position(target, options);
  },

  /**
   * Creates a signal that emits a value if the mouse state changes.
   *
   * When the mouse is moved or a button is pressed, then the signal will emit
   * an object representing the current state of the mouse.
   *
   * @param {EventTarget} target The event target (e.g. a DOM element).
   * @param {Object} [options] The options.
   * @param {Booelan} [options.preventDefault=false] A boolean indicating whether
   * the default action should be taken for the event.
   * @returns {Signal} A new signal.
   * @example
   *
   * import { mouse } from 'bulb-input'
   *
   * const s = Mouse.state(document)
   *
   * s.subscribe(console.log) // { buttons: 1, clientX: 1, clientY: 1, ... }, ...
   */
  state: function state$$1(target, options) {
    return state$1(target, options);
  }
};

exports.Keyboard = Keyboard;
exports.Mouse = Mouse;
